/*
	. Consider a set of processes arranged in a ring. 
	. Each processor stores its rank in MPI_COMM_WORLD in an integer and 
	sends this value onto the processor on its right. 
	. The processors continue passing on the values they receive 
	until they get their own rank back. Each process should finish 
	by printing out the sum of the values. 
*/

#include "mpi.h"

void main( int argc, char **argv )
{
	int value,			/* sending and return value to/from the process */
		total = 0,		/* total value */
		source, dest, tag = 0,
		my_rank, num_processes;
	double begin_time, stop_time;
	MPI_Request request1, request2;
	MPI_Status status;

	/* start up MPI
	*/
	MPI_Init( &argc, &argv );

	/* set up the starting time
	*/
	begin_time = MPI_Wtime();
  
	/* get process rank
	*/
	MPI_Comm_rank( MPI_COMM_WORLD, &my_rank );

	/* get number of processes
	*/
	MPI_Comm_size( MPI_COMM_WORLD, &num_processes );

	dest = ( my_rank + 1 ) % num_processes;
	source = ( my_rank - 1 + num_processes ) % num_processes;
	value = my_rank;

	/* Each process sends its rank to the next processor until they get their own rank back
	*/
	do {
		MPI_Isend( &value, 1, MPI_INT, dest, tag, MPI_COMM_WORLD, &request1 );
		MPI_Wait(&request1, &status);
		MPI_Irecv( &value, 1, MPI_INT, source, tag, MPI_COMM_WORLD, &request2 );
		MPI_Wait(&request2, &status);
		total += value;
	} while ( value != my_rank );

	/* stop the time
	*/
	stop_time = MPI_Wtime();

	/* print out the result
	*/
	printf( "Process_Rank Elapsed_Time Value\n" );
	printf( "%d %f %d\n", my_rank, stop_time - begin_time, total );

	/* close MPI
	*/
	MPI_Finalize();
}
